package com.ruoyi.project.VRdemo.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.framework.aspectj.lang.annotation.Anonymous;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.VRdemo.domain.LibraryMenu;
import com.ruoyi.project.VRdemo.service.ILibraryMenuService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 南洋数字图书馆菜单管理Controller
 *
 * @author ruoyi
 * @date 2024-11-01
 */
@RestController
@RequestMapping("/vr/libraryMenu")
public class LibraryMenuController extends BaseController {

    private final ILibraryMenuService libraryMenuService;

    public LibraryMenuController(@Autowired ILibraryMenuService libraryMenuService) {
        this.libraryMenuService = libraryMenuService;
    }

    /**
     * 查询南洋数字图书馆菜单管理列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @GetMapping("/list")
    public TableDataInfo list(LibraryMenu libraryMenu) {
        List<LibraryMenu> list = libraryMenuService.selectLibraryMenuList(libraryMenu);
        return getDataTable(list);
    }

    /**
     * 查询南洋数字图书馆一级菜单
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @GetMapping("/firstOrder")
    public AjaxResult firstOrder() {
        return success(libraryMenuService.selectMenuListByParentId(1L));
    }

    /**
     * 导出南洋数字图书馆菜单管理列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "南洋数字图书馆菜单管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, LibraryMenu libraryMenu) {
        List<LibraryMenu> list = libraryMenuService.selectLibraryMenuList(libraryMenu);
        ExcelUtil<LibraryMenu> util = new ExcelUtil<LibraryMenu>(LibraryMenu.class);
        util.exportExcel(response, list, "南洋数字图书馆菜单管理数据");
    }

    /**
     * 获取南洋数字图书馆菜单管理详细信息
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(libraryMenuService.selectLibraryMenuById(id));
    }

    /**
     * 新增南洋数字图书馆菜单管理
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "南洋数字图书馆菜单管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody LibraryMenu libraryMenu) {
        return toAjax(libraryMenuService.insertLibraryMenu(libraryMenu));
    }

    /**
     * 修改南洋数字图书馆菜单管理
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "南洋数字图书馆菜单管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody LibraryMenu libraryMenu) {
        return toAjax(libraryMenuService.updateLibraryMenu(libraryMenu));
    }

    /**
     * 删除南洋数字图书馆菜单管理
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "南洋数字图书馆菜单管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(libraryMenuService.deleteLibraryMenuByIds(ids));
    }

}
