package com.ruoyi.project.VRdemo.controller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.excel.EasyExcel;
import com.ruoyi.common.utils.excel.ExcelReadUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Anonymous;
import com.ruoyi.project.VRdemo.domain.imData.PublicationImData;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.VRdemo.domain.Publication;
import com.ruoyi.project.VRdemo.service.IPublicationService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 东南亚连续出版物Controller
 *
 * @author ruoyi
 * @date 2024-10-06
 */
@RestController
@RequestMapping("/vr/publication")
public class PublicationController extends BaseController {

    private final IPublicationService publicationService;

    public PublicationController(@Autowired IPublicationService publicationService) {
        this.publicationService = publicationService;
    }

    /**
     * 查询东南亚连续出版物列表
     */
    @Anonymous
    @GetMapping("/list")
    public AjaxResult list(Publication publication) {
       return publicationService.selectPublicationListByMenu(publication);
    }

    /**
     * 查询分页列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @GetMapping("/pageList")
    public TableDataInfo pageList(Publication publication) {
        startPage();
        List<Publication> list = publicationService.selectPublicationList(publication);
        return getDataTable(list);
    }

    /**
     * 获取东南亚连续出版物详细信息
     */
    @Anonymous
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(publicationService.selectPublicationById(id));
    }

    /**
     * 新增东南亚连续出版物
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "东南亚连续出版物", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody Publication publication) {
        return toAjax(publicationService.insertPublication(publication));
    }

    /**
     * 修改东南亚连续出版物
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "东南亚连续出版物", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody Publication publication) {
        return toAjax(publicationService.updatePublication(publication));
    }

    /**
     * 删除东南亚连续出版物
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "东南亚连续出版物", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(publicationService.deletePublicationByIds(ids));
    }

    /**
     * 导出东南亚连续出版物列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "东南亚连续出版物", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, Publication publication) {
        List<Publication> list = publicationService.selectPublicationList(publication);
        ExcelUtil<Publication> util = new ExcelUtil<>(Publication.class);
        util.exportExcel(response, list, "东南亚连续出版物数据");
    }

    /**
     * 下载导入模板
     */
    @Anonymous
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) throws IOException {
        List<PublicationImData> dataList = new ArrayList<>();
        EasyExcel.write(response.getOutputStream(), PublicationImData.class).sheet("Sheet1").doWrite(dataList);
    }

    /**
     * 导入数据
     */
    @Anonymous
    @PostMapping("/import")
    public AjaxResult importData(MultipartFile file, boolean updateSupport) throws IOException {

        List<Publication> list = ExcelReadUtil.read(file, PublicationImData.class, Publication.class);

        // 调用服务层方法进行数据导入处理
        String message = publicationService.importData(list, updateSupport);

        // 返回成功响应，携带导入结果信息
        return AjaxResult.success(message);
    }
}
