package com.ruoyi.project.VRdemo.qiniu;

import com.google.gson.Gson;
import com.qiniu.common.QiniuException;
import com.qiniu.http.Response;
import com.qiniu.storage.*;
import com.qiniu.storage.model.DefaultPutRet;
import com.qiniu.storage.persistent.FileRecorder;
import com.qiniu.util.Auth;
import com.ruoyi.framework.web.domain.AjaxResult;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Date;

@Service
public class QiniuService {

    @Value("${qiniu.accessKey}")
    private String accessKey;

    @Value("${qiniu.secretKey}")
    private String secretKey;

    @Value("${qiniu.bucket}")
    private String bucket;

    @Value("${qiniu.domain}")
    private String domain;

    @Value("${qiniu.zone}")
    private String filePath;

    /**
     * 普通上传（不覆盖文件，可修改）
     * @param file
     * @return
     */
    public AjaxResult upload(MultipartFile file) {
        Configuration cfg = new Configuration();
        UploadManager uploadManager = new UploadManager(cfg);

        Auth auth = Auth.create(accessKey, secretKey);
        // 生成上传凭证
        String uploadToken = auth.uploadToken(bucket);

        // 调用put方法上传
        // 指定文件的存储路径（键）
        String key = filePath + file.getOriginalFilename();
        String name = key;
        int lastSlashIndex = name.lastIndexOf('/');
        if (lastSlashIndex != -1){
            name = name.substring(lastSlashIndex + 1);
        }

        try {
            Response response = uploadManager.put(file.getBytes(), key, uploadToken);
            // 解析上传结果
            if (response.isOK()) {
                AjaxResult ajax = AjaxResult.success();
                ajax.put("url", domain + "/" + key);
                ajax.put("fileName", key);
                ajax.put("newFileName", name);
                ajax.put("originalFilename", file.getOriginalFilename());
                return ajax;
            } else {
                throw new QiniuException(response);
            }
        } catch (QiniuException e) {
            // 处理上传异常
            e.printStackTrace();
            // 返回错误信息或抛出自定义异常
        } catch (IOException e) {
            // 处理文件读取异常
            e.printStackTrace();
            // 返回错误信息或抛出自定义异常
        }
        return null;
    }

    /**
     * 删除文件
     * @param url
     * @throws QiniuException
     */
    public AjaxResult deleteFile(String url) throws QiniuException {
        Configuration cfg = new Configuration();
        Auth auth = Auth.create(accessKey, secretKey);

        BucketManager bucketManager = new BucketManager(auth, cfg);

        //获取文件的key
        String key = url;
        int index = key.indexOf('/');
        if (index != -1){
            key = key.substring(index + 1);
        }
        // 删除文件
        System.out.println(url);
        System.out.println(key);
        Response response = bucketManager.delete(bucket, key);

        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.put("flag",response);
        return ajaxResult;
    }

    /**
     * 断点上传，（覆盖上传，可修改）
     * @param file
     * @return
     */
    public AjaxResult breakpointUpload(MultipartFile file, String directory) {
//        Configuration cfg = new Configuration(Region.region0());
        Configuration cfg = new Configuration();
        cfg.resumableUploadAPIVersion = Configuration.ResumableUploadAPIVersion.V2;// 指定分片上传版本
        cfg.resumableUploadMaxConcurrentTaskCount = 2;  // 设置分片上传并发，1：采用同步上传；大于1：采用并发上传

        Auth auth = Auth.create(accessKey, secretKey);

        //时间格式化格式
        SimpleDateFormat simpleDateFormat =new SimpleDateFormat("yyyyMMddHHmmssSSS");
        //获取当前时间并作为时间戳给文件夹命名
        String timeStamp = simpleDateFormat.format(new Date());

        // 生成上传凭证
        String key = filePath + directory + timeStamp + file.getOriginalFilename();
        String uploadToken = auth.uploadToken(bucket,key);

        String localTempDir = Paths.get(System.getenv("java.io.tmpdir"), bucket).toString();
        try {
            //设置断点续传文件进度保存目录
            FileRecorder fileRecorder = new FileRecorder(localTempDir);
            UploadManager uploadManager = new UploadManager(cfg, fileRecorder);
            try {
                Response response = uploadManager.put(file.getBytes(), key, uploadToken);
                //解析上传成功的结果
                DefaultPutRet putRet = new Gson().fromJson(response.bodyString(), DefaultPutRet.class);

                String name = putRet.key;
                int lastSlashIndex = name.lastIndexOf('/');
                if (lastSlashIndex != -1){
                    name = name.substring(lastSlashIndex + 1);
                }

                System.out.println("储存位置: " + putRet.key);//储存位置
                System.out.println("文件名: " + name);//文件名
                System.out.println("完整的访问路径: " + domain + "/" + key);//完整的访问路径

                AjaxResult ajax = AjaxResult.success();
                ajax.put("url", domain + "/" + key);
                //ajax.put("fileName", domain + "/" + key);
                ajax.put("fileName", putRet.key);
                ajax.put("newFileName", name);
                ajax.put("originalFilename", file.getOriginalFilename());
                return ajax;
            } catch (QiniuException ex) {
                ex.printStackTrace();
                if (ex.response != null) {
                    System.err.println(ex.response);
                    try {
                        String body = ex.response.toString();
                        System.err.println(body);
                    } catch (Exception ignored) {
                    }
                }
            }

        } catch (IOException ex) {
            ex.printStackTrace();
        }
        return null;
    }

    /**
     * 切片上传（未完成
     * @param multipartFile
     * @return
     */
    public String sliceUpload(MultipartFile multipartFile) {
        //构造一个带指定 Region 对象的配置类
        Configuration cfg = new Configuration(Region.region0());
        cfg.resumableUploadAPIVersion = Configuration.ResumableUploadAPIVersion.V2;// 指定分片上传版本
        cfg.resumableUploadMaxConcurrentTaskCount = 2;  // 设置分片上传并发，1：采用同步上传；大于1：采用并发上传

        return null;
    }

}
