package com.ruoyi.project.VRdemo.service.impl;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.framework.config.RuoYiConfig;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.project.VRdemo.domain.FileUpload;

import com.ruoyi.project.VRdemo.mapper.FileUploadMapper;

import com.ruoyi.project.VRdemo.service.IFileUploadService;

import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
@Service
public class FileUploadServiceImpl implements IFileUploadService
{
    @Autowired
    private FileUploadMapper fileUploadMapper;
    /**
     * 查询data
     *
     * @param fileid data主键
     * @return data
     */
    @Override
    public FileUpload selectFileUploadByfileid(Long fileid) {
        return fileUploadMapper.selectFileUploadByfileid(fileid);
    }
    /**
     * 查询data列表
     *
     * @param fileUpload data
     * @return data
     */
    @Override
    public List<FileUpload> selectFileUploadList(FileUpload fileUpload) {
        return fileUploadMapper.selectFileUploadList(fileUpload);
    }
    /**
     * 新增data
     *
     * @param fileUpload data
     * @return 结果
     */
    @Override
    public int insertFileUpload(FileUpload fileUpload) {
       fileUpload.setCreateTime(DateUtils.getNowDate());
        return fileUploadMapper.insertFileUpload(fileUpload);
    }
    /**
     * 修改data
     *
     * @param fileUpload data
     * @return 结果
     */
    @Override
    public int updateFileUpload(FileUpload fileUpload) {
        fileUpload.setUpdateTime(DateUtils.getNowDate());
        return fileUploadMapper.updateFileUpload(fileUpload);
    }
    /**
     * 删除data信息
     *
     * @param fileid data主键
     * @return 结果
     */
    @Override
    public int deleteFileUploadByfileid(Long fileid) {
        return fileUploadMapper.deleteFileUploadfileid(fileid);
    }
    /**
     * 新增【文件】
     *
     * @param file 【文件】
     * @return 结果
     */
    @Override
    public String  uploadFile(MultipartFile file, FileUpload fileUpload) throws Exception    {
        String extension = FilenameUtils.getExtension(file.getOriginalFilename());
        String filePath;
        if (extension.equalsIgnoreCase("pdf")) {
            filePath = RuoYiConfig.getuploadPdf();
              return FileUploadUtils.upload(filePath, file);
        } else if (extension.equalsIgnoreCase("doc") || extension.equalsIgnoreCase("docx")) {
            filePath = RuoYiConfig.getuploadWord();
            return FileUploadUtils.upload(filePath, file);
        } else {
            filePath = RuoYiConfig.getUploadPath();
            return FileUploadUtils.upload(filePath, file);
        }
    }
    //删除
    public boolean deleteFile(Long fileid) {
        // 根据ID查询文件信息
        FileUpload file = fileUploadMapper.selectFileUploadByfileid(fileid);
        System.out.println("4." + file);
        if (file == null) {
            return false; // 文件不存在，删除失败
        }

        String oldPath=file.getPath();
        if(oldPath!=null){
            String filePath = RuoYiConfig.getUploadPath();
            String deletaPath =filePath + StringUtils.substringAfter(oldPath, Constants.RESOURCE_PREFIX);
            System.out.println("5." + deletaPath);
        if (deleteFileRecursive(deletaPath)) {
            // 执行特定文件夹删除操作
            String fileExtension = getFileExtensionFromPath(deletaPath);
            if (fileExtension.equalsIgnoreCase("pdf")) {
                String subFolderPath = getSubFolderPath(deletaPath, "pdf");
                System.out.println("8." + subFolderPath);
                deleteFolder(subFolderPath); // 修改这一行
            } else if (fileExtension.equalsIgnoreCase("doc") || fileExtension.equalsIgnoreCase("docx")) {
                String subFolderPath = getSubFolderPath(deletaPath, "word");
                System.out.println("9." + subFolderPath);
                deleteFolder(subFolderPath); // 修改这一行
            }

            // 删除数据库中的文件记录
            int deleteCount = fileUploadMapper.deleteFileUploadfileid(fileid);
            System.out.println("文件删除成功");
            return true; // 文件删除成功
        } else {
            System.out.println("文件删除失败");
            return false; // 文件删除失败
        }
        }
        return false;
    }
//只删除文件
public boolean updateOnlyFile(Long fileid) {
    // 根据ID查询文件信息
    String newFile = "";
    FileUpload file = fileUploadMapper.selectFileUploadByfileid(fileid);
    String oldFile = file.getPath();
    if (oldFile != null) {
        String filePath = RuoYiConfig.getProfile();
        String deletaPath = filePath + StringUtils.substringAfter(oldFile, Constants.RESOURCE_PREFIX);
        if (FileUtils.deleteFile(deletaPath)) {
            boolean flag = fileUploadMapper.updateFileUploadUrlData(newFile, fileid);
            if (flag) {
                return true;
            }
        }
    }
    return false;
}

    // 从文件路径中获取文件扩展名
    private String getFileExtensionFromPath(String filePath) {
        int dotIndex = filePath.lastIndexOf(".");
        if (dotIndex >= 0 && dotIndex < filePath.length() - 1) {
            return filePath.substring(dotIndex + 1).toLowerCase();
        }
        System.out.println("10."+dotIndex);
        return "";
    }

    // 根据文件路径和指定的文件夹名称获取子文件夹路径
    private String getSubFolderPath(String filePath, String folderName) {
        int index = filePath.lastIndexOf(File.separator + folderName + File.separator);
        if (index != -1) {
            return filePath.substring(0, index);
        }
        return filePath;
    }



    /**
     * 删除文件
     *
     * @param fileid 文件
     * @return
     */
    public static boolean deleteFileRecursive(String fileid)
    {
        File file = new File(fileid);
        System.out.println("11."+file.getPath());
        if (file.isFile() && file.exists()) {
            return file.delete();
        }
        System.out.println(false);
        return false;
    }
    // 递归删除文件夹及其内容
    private void deleteFolder(String filePath) {
        File folder = new File(filePath);
        System.out.println("12." + folder);
        if (folder.exists()) {
            File[] files = folder.listFiles();
            if (files != null) {
                for (File file : files) {
                    if (file.isDirectory()) {
                        deleteFolder(file.getPath()); // 使用完整的文件夹路径
                    } else {
                        file.delete();
                    }
                }
                folder.delete();
            }
        }
    }

}

